{% load i18n %}
<script type="text/javascript">
    window.stream_texts = window.stream_texts || {
        'deleteBlock': '{% trans "Are you shure, that you want to delete this block?" %}',
        'deleteInstance': '{% trans "Are you shure, that you want to delete this subblock?" %}',
        'Collapse all': '{% trans "Collapse all" %}',
        'Open all': '{% trans "Open all" %}',
        'AddOneMore': '{% trans "Add one more" %}',
        'Change': '{% trans "Change" %}',
        'AddContent': '{% trans "Add content" %}',
        'Help?': '{% trans "Help?" %}',
        'AddNewBlock': '{% trans "Add new block" %}',
        'help_text': "{{ widget.value.help_text|escapejs|default:"" }}"
    }
</script>
<div class="streamfield_app" id="app_{{ widget.name }}">
    <div class="mount-node"></div>
    <textarea v-text="textarea" name="{{ widget.name }}"{% include "django/forms/widgets/attrs.html" %}>{% if widget.value %}{{ widget.value }}{% endif %}</textarea>
</div>